"""
VendAsta Package Manager.
"""
import json

class Package(object):
    """
    Representation of a single package. Corresponds to the JSON object.
    {
        "name": "vapi",
        "installPath": "src/lib/",
        "githubPath": "VA/vapi/src/vapi"
    }
    """

    NAME = 'name'
    INSTALL_PATH = 'installPath'
    GITHUB_PATH = 'githubPath'
    CACHE_ROOT = '.vpmcache'

    def __init__(self, **kwargs):
        """
        Initialize.
        """
        self.name = kwargs.get(Package.NAME)
        self.install_path = kwargs.get(Package.INSTALL_PATH)
        self.github_path = kwargs.get(Package.GITHUB_PATH)

        if not self.name:
            raise ValueError('%s is required.', Package.NAME)
    
        if not self.install_path:
            raise ValueError('%s is required.', Package.INSTALL_PATH)

        if not self.github_path:
            raise ValueError('%s is required.', Package.GITHUB_PATH)

    @property
    def repository(self):
        """
        Returns the repository from the github_path.
        """
        return self.github_path.split('/', 1)[0]

    @property
    def directory_path(self):
        """
        Returns the directory from the github_path.
        """
        return self.github_path.split('/', 1)[1]

    @property
    def download_path(self):
        """
        Returns the directory to download to.
        """
        return Package.CACHE_ROOT + '/' + self.name


class Packages(object):
    """
    Representation of a list of packages.
    """

    FILENAME = 'packages.json'

    @classmethod
    def from_file(cls, filename=None):
        """
        Initialize from filename.
        """
        filename = filename or Packages.FILENAME

        packages = open(filename, "r")
        json_data = packages.read()
        data = json.loads(json_data)

        return [Package(**p) for p in data]


class Configuration(object):
    """
    Representation of a vpm configuration. Corresponds to the following JSON object.
    {
        "githubUsername": "ksookocheff-va",
        "githubPassword": "1234567890",
    }
    """

    FILENAME = 'vpmrc.json'

    GITHUB_USERNAME = 'githubUsername'
    GITHUB_PASSWORD = 'githubPassword'


    def __init__(self, **kwargs):
        """
        Initialize.
        """
        self.username = kwargs.get(Configuration.GITHUB_USERNAME)
        self.password = kwargs.get(Configuration.GITHUB_PASSWORD)

        if not self.username:
            raise ValueError('%s is required.', Configuration.GITHUB_USERNAME)
    
        if not self.password:
            raise ValueError('%s is required.', Configuration.GITHUB_PASSWORD)

    @classmethod
    def from_file(cls, filename=None):
        """
        Initialize from filename.
        """
        filename = filename or Configuration.FILENAME

        config_file = open(filename, "r")
        json_data = config_file.read()
        data = json.loads(json_data)

        return cls(**data)

